@@ -26,6 +26,7 @@ class Controller(ABC):
         self.__time_dataset: List[float] = []
         self.__error_dataset: List[float] = []
         self.__cost_dataset: List[float] = []
+        self.__is_call_exact: List[int] = []  # 1 if the exact model has been called
         self.__to_reshape = False
         self._exact: Callable[[List[float]], List[float]] = exact_fitness
         self._estimator: Estimator = estimator
@@ -47,7 +48,9 @@ class Controller(ABC):
         mab_forced_probability: Optional[int] = None,
         mab_weight: Optional[MabHandler] = None,
         mab_weight_forced_probability: Optional[int] = None,
+        is_point_in_context: bool = True,
     ) -> Tuple[List[float], float]:
+        self.__is_call_exact.append(1)
         if self._self_exact:
             exact, exec_time = Controller.measure_execution_time(
                 self._exact, self._self_exact, point
@@ -58,19 +61,23 @@ class Controller(ABC):
         self._estimator.train(point, exact)
         if mab:
             if mab_weight:
-                time_weight = mab_weight.predict([mab[0].get_mean_cost()]) / 100
+                time_weight = (
+                    mab_weight.predict(self._compute_weighting_context()) / 100
+                )
                 mab[0].learn(
                     mab[1],
                     self.get_time_error_z_score(
                         exec_time, 0, *(time_weight, 1 - time_weight)
                     ),
-                    point,
+                    self._compute_controller_context(
+                        point if is_point_in_context else None
+                    ),
                     mab_forced_probability,
                 ),
                 mab_weight.learn(
                     time_weight * 100,
                     self.get_time_error_correlation(),
-                    [mab[0].get_mean_cost()],
+                    self._compute_weighting_context(),
                     mab_weight_forced_probability,
                 )
             else:
@@ -80,24 +87,46 @@ class Controller(ABC):
                         exec_time,
                         0,
                     ),
-                    point,
+                    self._compute_controller_context(
+                        point if is_point_in_context else None
+                    ),
                     mab_forced_probability,
                 )
         return exact, exec_time
 
+    def _compute_controller_context(self, point: List[int] = None) -> List[float]:
+        if not point:
+            return [
+                self._estimator.get_error(),
+                np.mean(self.__time_dataset) if len(self.__time_dataset) > 0 else 0,
+            ]
+        else:
+            return [
+                self._estimator.get_error(),
+                np.mean(self.__time_dataset) if len(self.__time_dataset) > 0 else 0,
+            ] + point
+
+    def _compute_weighting_context(self) -> List[float]:
+        estimation_calls = self.__is_call_exact.count(0)
+        return [estimation_calls, len(self.__is_call_exact) - estimation_calls]
+
     def _compute_estimated(
         self,
         point: List[int],
         mab: Optional[Tuple[MabHandler, Union[int, float]]] = None,
         mab_weight: MabHandler = None,
+        is_point_in_context: bool = True,
     ) -> Tuple[List[float], float]:
+        self.__is_call_exact.append(0)
         estimation, exec_time = Controller.measure_execution_time(
             self._estimator.predict,
             point,
         )
         if mab:
             if mab_weight:
-                time_weight = mab_weight.predict([mab[0].get_mean_cost()]) / 100
+                time_weight = (
+                    mab_weight.predict(self._compute_weighting_context()) / 100
+                )
                 mab[0].learn(
                     mab[1],
                     self.get_time_error_z_score(
@@ -105,12 +134,14 @@ class Controller(ABC):
                         self._estimator.get_error(),
                         *(time_weight, 1 - time_weight)
                     ),
-                    point,
+                    self._compute_controller_context(
+                        point if is_point_in_context else None
+                    ),
                 )
                 mab_weight.learn(
                     time_weight,
                     self.get_time_error_correlation(),
-                    [mab[0].get_mean_cost()],
+                    self._compute_weighting_context(),
                 )
             else:
                 mab[0].learn(
@@ -119,7 +150,9 @@ class Controller(ABC):
                         exec_time,
                         self._estimator.get_error(),
                     ),
-                    point,
+                    self._compute_controller_context(
+                        point if is_point_in_context else None
+                    ),
                 )
 
         return estimation, exec_time
