@@ -50,7 +50,7 @@ class MabController(Controller):
         )
 
     def compute_objective(self, point: List[int]) -> List[float]:
-        decision = self.__mab.predict([*point, self._estimator.get_error()])
+        decision = self.__mab.predict(self._compute_controller_context(point))
         accuracy = self._estimator.get_error()
         if decision == 2 or accuracy == 0.0:
             out, exec_time = self._compute_exact(
